from sys import argv, stdin, exit
from re import match
import sys
#import pdb

EASY=0
HARD=1
IMP = "impossible"

assert len(argv)==5, "correct usage: python3 %s 0/1 input our_output contestants_output" % argv[0]
assert int(argv[1]) in [ EASY, HARD ], "the first argument should be %s (=EASY) or %s (=HARD)" % (EASY, HARD)
DIFFICULTY = int(argv[1])

try:
    try:    # Reading input
        fin = open(argv[2],'r')
        n = int(fin.readline())
        in_tests = [ t.split() for t in fin.readlines() if t.strip() ]
        in_tests = [ ( int(t[0]), int(t[1]), str(t[2]) ) for t in in_tests ]
        assert len(in_tests) == n
        assert all([ t[2] in ['B', 'W'] for t in in_tests ])
    except Exception:
        sys.exit("Something is wrong, contact organizers. (Error 3287)")

    try:    # Reading our output
        our_output = open(argv[3], 'r').read().split()
        assert all([ (o.isdigit() and int(o) >= 1) or o == IMP for o in our_output ])
    except Exception:
        sys.exit("Something is wrong, contact organizers. (Error 1290)")

    try:    # Reading their output
        their_output = open(argv[4], 'r', encoding='ascii').read().split()
    except Exception:
        assert False, "Wrong answer: Failed to parse the submission. (Binary file?)"

    assert all([t.isdigit() or t == IMP for t in their_output]), "Wrong answer: The answer must contain only integers and 'impossible'."

    their_pos = 0
    our_pos = 0

    for tid, test in enumerate(in_tests):
        R, C, color = test
        path_len = (R * C + (color == 'W')) // 2         # number of cells of color 'color'

        # Parsing our testcase output:
        if len(our_output) - our_pos < 1: sys.exit("Something is wrong, contact organizers. (Error 6329)")
        if str(our_output[our_pos]) == IMP:
            is_valid_sol = False
            our_pos += 1
        else:
            if len(our_output) - our_pos < 2 * path_len: sys.exit("Something is wrong, contact organizers. (Error 9732)")
            is_valid_sol = True
            our_pos += 2*path_len

        # Parsing their testcase output
        if not is_valid_sol:
            assert len(their_output) - their_pos >= 1, "Wrong answer"
            assert their_output[their_pos] == IMP, "Wrong answer"
            their_pos += 1

        else:
            assert len(their_output) - their_pos >= 1, "Wrong answer"
            assert their_output[their_pos] != IMP, "Wrong answer"

            assert len(their_output) - their_pos >= 2*path_len, "Wrong answer"

            testcase = their_output[their_pos:their_pos+2*path_len]
            their_pos += 2*path_len
            #if tid == 71:           pdb.set_trace()

            assert all([t.isdigit() for t in testcase]), "Wrong answer"

            testcase = [ ( int(testcase[2*i]), int(testcase[2*i+1]) ) for i in range(path_len) ]

            # check cells in board
            assert all([ 1 <= r <= R and 1 <= c <= C for (r, c) in testcase ]), "Wrong answer"
            # check cells color
            assert all([ (r+c)%2 == (color=='B') for (r, c) in testcase ]), "Wrong answer"

            # check cell uniqueness
            assert len(testcase) == len(set(testcase)), "Wrong answer"

            # check that cells are neighboring
            for i in range(len(testcase)-1):
                dr = abs(testcase[i][0]) - abs(testcase[i+1][0])
                dc = abs(testcase[i][1]) - abs(testcase[i+1][1])
               
                if DIFFICULTY == EASY:
                    assert abs(dr) == abs(dc) and abs(dr)>= 1, "Wrong answer"
                if DIFFICULTY == HARD:
                    assert abs(dr) == abs(dc) and abs(dr)== 1, "Wrong answer"

except Exception as error_message:
    print(error_message)
    exit(1)

print("OK")

